@@ -1,13 +1,13 @@
 profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Customized model", neg=TRUE, alpha=0.05)
 {
+        X<-as.matrix(X)
+        t <- ifelse(dfreq,dim(X)[2]-1,dim(X)[2])
+
         #####################################################################################################################################
         # Validation des arguments fournis en entre
-        
+
         # Argument dfreq
-        if(!is.logical(dfreq)||length(dfreq)!=1) stop("'dfreq' must be a logical object of length 1")
-    
-            X <- as.matrix(X)
-            t <- if(dfreq) dim(X)[2]-1 else dim(X)[2]
+        if(!is.logical(dfreq)||!isTRUE(all.equal(length(dfreq),1))) stop("'dfreq' must be a logical object of length 1")
     
         # Argument X
         if (dfreq)
@@ -19,39 +19,39 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
         }
        
         # Argument m
-        if(length(m)!=1) stop("'m' must be of length 1")
-        if(m!="M0"&&m!="Mt"&&m!="Mh"&&m!="Mth")
+        if(!isTRUE(all.equal(length(m),1))) stop("'m' must be of length 1")
+        if(!identical(m,"M0")&&!identical(m,"Mt")&&!identical(m,"Mh")&&!identical(m,"Mth"))
         stop("'m' can only take the value 'M0', 'Mt', 'Mh' or 'Mth'")
     
         # Argument h
-        if(length(h)!=1) stop("'h' must be of length 1")
-        if(!is.function(h)&&h!="Chao"&&h!="Darroch"&&h!="Poisson")
+        if(!isTRUE(all.equal(length(h),1))) stop("'h' must be of length 1")
+        if(!is.function(h)&&!identical(h,"Chao")&&!identical(h,"Darroch")&&!identical(h,"Poisson"))
         stop("'h' must be a function or a charater strings taking the value 'Chao', 'Darroch' or 'Poisson'")
         
         # Argument a
-        if(length(a)!=1) stop("'a' must be of length 1")
+        if(!isTRUE(all.equal(length(a),1))) stop("'h' must be of length 1")
         if (!is.numeric(a)) stop("'a' must be a numeric value")
     
         # Argument mX
-        if(!is.null(mX))
+        if(!isTRUE(all.equal(mX,NULL)))
         {
             mX<-as.matrix(mX)
-            if (dim(mX)[1]!=2^t-1) stop("'mX' must have 2^t-1 rows")
+            if (!isTRUE(all.equal(2^t-1,dim(mX)[1]))) stop("'mX' must have 2^t-1 rows")
         }
 
         # Argument neg
-        if(!is.logical(neg)||length(neg)!=1) stop("'neg' must be a logical object of length 1")
+        if(!is.logical(neg)||!isTRUE(all.equal(length(neg),1))) stop("'neg' must be a logical object of length 1")
 
         #####################################################################################################################################
 
         
         Y <- histfreq.t(X,dfreq=dfreq)
-        mXsans <- if (is.null(mX)) Xclosedp(t,m,h,a)$mat else mX
+        mXsans <- if (isTRUE(all.equal(mX,NULL))) Xclosedp(t,m,h,a)$mat else mX
         rd.call <- match.call()
                       
 ######### Obtention de l'estimateur de N (sans mu_0). Note : c'est approximatif, on n'ajuste donc pas pour les eta ngatifs modles Chao
         anasans <- glm(Y~mXsans,family=poisson)
-        Np <- sum(na.rm=TRUE,Y)+exp(anasans$coef[1])
+        Np <- sum(Y)+exp(anasans$coef[1])
         varcov <- summary(anasans)$cov.unscaled
         SEp <- sqrt(exp(anasans$coef[1])+(exp(2*anasans$coef[1]))*varcov[1,1])
                                
@@ -68,7 +68,7 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
 
 
 ######### Dtermination des 21 points de dpart
-        saut <- round((Np-sum(na.rm=TRUE,Y)+2*qnorm(1-alpha/2)*SEp)/20)
+        saut <- round((Np-sum(Y)+2*qnorm(1-alpha/2)*SEp)/20)
         points <- (0:20)*saut
         
 ######### Boucle de calcul de la vraisemblance multinomiale profile
@@ -88,36 +88,36 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
                     # Vrification eta ngatifs pour modles Chao
                     if(neg)
                     {
-                        if ((m=="Mh"||m=="Mth")&&h=="Chao")
+                        if ((identical(m,"Mh")||identical(m,"Mth"))&&identical(h,"Chao"))
                         {
                             # Rajustement du modle en enlevant les eta ngatifs
                             ppositions <- 0
                             param <- anaavec$coef
                             indic <- as.vector(c(rep(0,length(param)-t+2),ifelse(param[(length(param)-t+3):length(param)]<0,1,0)))
-                            while(sum(na.rm=TRUE,indic)>0) # Rpter la boucle jusqu' ce qu'aucun eta ne soit ngatif
+                            while(isTRUE(sum(indic)>0)) # Rpter la boucle jusqu' ce qu'aucun eta ne soit ngatif
                             {
                                 # Dtermination de la position du premier eta ngatif
                                 pos <- 1
-                                while(indic[pos]==0) pos <- pos + 1
+                                while(isTRUE(all.equal(indic[pos],0))) pos <- pos + 1
                                 ppositions <- c(ppositions,pos)
                                 # Retrait de la bonne colonne de mX et rajustement du modle
-                                mXavec <- matrix(mXavec[,-(pos-sum(na.rm=TRUE,ppositions<pos))],nrow=dim(mXavec)[1])
+                                mXavec <- matrix(mXavec[,-(pos-sum(ppositions<pos))],nrow=dim(mXavec)[1])
                                 anaavec <- glm(Yavec~mXavec,family=poisson)
                                 # Ajout de zros dans le vecteur des paramtres loglinaires
                                 positions <- sort(ppositions[-1])                
                                 param <- c(anaavec$coef[1:(positions[1]-1)],0)
-                                if(length(positions)>1)
+                                if(isTRUE(length(positions)>1))
                                 {
                                     for ( i in 2:length(positions))
                                     {
-                                        if(positions[i]==positions[i-1]+1) {
+                                        if(isTRUE(all.equal(positions[i],positions[i-1]+1))) {
                                             param <- c(param,0)
                                         } else {
                                             param <- c(param,anaavec$coef[(positions[i-1]-i+2):(positions[i]-i)],0)
                                         }
                                     }
                                 }
-                                if((positions[length(positions)]-length(positions)+1)<=length(anaavec$coef)) param <- c(param,anaavec$coef[(positions[length(positions)]-length(positions)+1):length(anaavec$coef)])                   
+                                if( (positions[length(positions)]-length(positions)+1) <= length(anaavec$coef)) param <- c(param,anaavec$coef[(positions[length(positions)]-length(positions)+1):length(anaavec$coef)])                   
                                 indic <- as.vector(c(rep(0,length(param)-t+2),ifelse(param[(length(param)-t+3):length(param)]<0,1,0)))
                             }
                         }
@@ -125,10 +125,10 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
                     #############################################################################################################
                         
                     # calcule du terme correctif
-                    Nn0 <- sum(na.rm=TRUE,Yavec)
-                    if(Nn0>100)
+                    Nn0 <- sum(Yavec)
+                    if(isTRUE(Nn0>100))
                     {
-                    ct <- if (n0==0||n0==1) -Nn0+0.5*log(2*pi*Nn0) else n0-Nn0-0.5*log(n0/Nn0)
+                    ct <- if (isTRUE(all.equal(n0,0))||isTRUE(all.equal(n0,1))) -Nn0+0.5*log(2*pi*Nn0) else n0-Nn0-0.5*log(n0/Nn0)
                     } else { ct <- log((n0^n0)*factorial(Nn0)/((Nn0^Nn0)*factorial(n0))) } 
                     
                     # log vraisemblance multinomiale profile
@@ -137,7 +137,7 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
             loglik <- loglik[-1]
                               
             # Pour former le vecteur complet de log vraisemblance profile
-            if(iter==1)
+            if(isTRUE(all.equal(iter,1)))
             {
                 loglik_c <- loglik
                 points_c <- points
@@ -154,7 +154,9 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
             
             # Dtermination du maximum et des points suivants
             lmax <- max(loglik_c)
-            pos_lmax <- which.max(loglik_c)
+            pos_lmax <- 1
+            while(loglik_c[pos_lmax]<lmax) { pos_lmax <- pos_lmax + 1 }
+            
             saut <- ceiling(saut/10)
             points <- c(points_c[pos_lmax]-(9:1)*saut,points_c[pos_lmax]+(1:9)*saut)
             points <- points[points>=0]
@@ -163,7 +165,7 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
 
 
 ######### Dterminiation du N qui maximise la log vraisemblance multinomiale profile 
-        x <- sum(na.rm=TRUE,Y)+points_c
+        x <- sum(Y)+points_c
         N <- x[pos_lmax]
         
 ######### Interpolation linaire pour trouver la valeur approximative des bornes de l'intervalle
@@ -173,12 +175,12 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
         posInfMin<-posInfMax-1
         posSupMax<-posInfMax
         while(sup[posSupMax]) { posSupMax<-posSupMax+1 }
-        InfCL <- if(posInfMax==1) sum(na.rm=TRUE,Y) else x[posInfMax-1]+((lmax-qchisq(1-alpha,1)/2-loglik_c[posInfMax-1])/(loglik_c[posInfMax]-loglik_c[posInfMax-1]))*(x[posInfMax]-x[posInfMax-1])
+        InfCL <- if(isTRUE(all.equal(posInfMax,1))) sum(Y) else x[posInfMax-1]+((lmax-qchisq(1-alpha,1)/2-loglik_c[posInfMax-1])/(loglik_c[posInfMax]-loglik_c[posInfMax-1]))*(x[posInfMax]-x[posInfMax-1])
         SupCL <- x[posSupMax-1]+(loglik_c[posSupMax-1]-(lmax-qchisq(1-alpha,1)/2))/(loglik_c[posSupMax-1]-loglik_c[posSupMax])*(x[posSupMax]-x[posSupMax-1])
         
 ######### Production du graphique
         plot(x[posInfMin:posSupMax],loglik_c[posInfMin:posSupMax],type="l",ylab="multinomial profile loglikelihood",xlab="N",main="Profile Likelihood Confidence Interval")
-        lInf <- if (InfCL==sum(na.rm=TRUE,Y)) loglik_c[1] else lmax-qchisq(1-alpha,1)/2
+        lInf <- if (isTRUE(all.equal(InfCL,sum(Y)))) loglik_c[1] else lmax-qchisq(1-alpha,1)/2
         segments(x0=InfCL,y0=min(loglik_c[posInfMin:posSupMax]),x1=InfCL,y1=lInf)
         text(InfCL,min(min(loglik_c[posInfMin:posSupMax])),round(InfCL,2),pos=1,offset=0.2)
         segments(x0=SupCL,y0=min(loglik_c[posInfMin:posSupMax]),x1=SupCL,y1=lmax-qchisq(1-alpha,1)/2)
@@ -188,10 +190,10 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
 
 ######### Production sortie non graphique
         results<-matrix(c(N,InfCL,SupCL),nrow=1)
-        mname<- if(!is.null(mX)) mname else if (m=="M0"||m=="Mt") m else if (is.function(h)) paste(m,rd.call$h) else if(h=="Poisson") paste(m,paste(h,a,sep="")) else paste(m,h)
+        mname<- if(!isTRUE(all.equal(mX,NULL))) mname else if (identical(m,"M0")||identical(m,"Mt")) m else if (is.function(h)) paste(m,rd.call$h) else if(identical(h,"Poisson")) paste(m,paste(h,a,sep="")) else paste(m,h)
         dimnames(results) <- list(mname,c("abundance","InfCL","SupCL"))
 
-        ans <- list(n=sum(na.rm=TRUE,Y),results=results,alpha=alpha)
+        ans <- list(n=sum(Y),results=results,alpha=alpha)
         class(ans) <- "profileCI"
         ans
 
@@ -200,7 +202,7 @@ profileCI <- function(X, dfreq=FALSE, m="M0", h="Chao", a=2, mX=NULL, mname="Cus
 
 print.profileCI <- function(x, ...) {
         cat("\nNumber of captured units:",x$n,"\n\n")
-        cat(paste((1-x$alpha)*100,"%",sep=""),"Profile likelihood confidence interval:\n")
+        cat(paste((1-x$alpha)*100,"%",sep=""),"profile likelihood confidence interval:\n")
         print.default(x$results, print.gap = 2, quote = FALSE, right=TRUE)
         cat("\n")
         invisible(x)
